@@ -11,6 +11,7 @@ import (
 
 	"github.com/AlecAivazis/survey/v2"
 	"github.com/containerd/containerd/identifiers"
+	"github.com/lima-vm/lima/cmd/limactl/editflags"
 	"github.com/lima-vm/lima/cmd/limactl/guessarg"
 	"github.com/lima-vm/lima/pkg/editutil"
 	"github.com/lima-vm/lima/pkg/ioutilx"
@@ -27,6 +28,52 @@ import (
 	"github.com/spf13/cobra"
 )
 
+func registerCreateFlags(cmd *cobra.Command, commentPrefix string) {
+	flags := cmd.Flags()
+	// TODO: "survey" does not support using cygwin terminal on windows yet
+	flags.Bool("tty", isatty.IsTerminal(os.Stdout.Fd()), commentPrefix+"enable TUI interactions such as opening an editor, defaults to true when stdout is a terminal")
+	flags.String("name", "", commentPrefix+"override the instance name")
+	flags.Bool("list-templates", false, commentPrefix+"list available templates and exit")
+	editflags.RegisterCreate(cmd, commentPrefix)
+}
+
+func newCreateCommand() *cobra.Command {
+	var createCommand = &cobra.Command{
+		Use: "create FILE.yaml|URL",
+		Example: `
+To create an instance "default" from the default Ubuntu template:
+$ limactl create
+
+To create an instance "default" from a template "docker":
+$ limactl create --name=default template://docker
+
+To create an instance "default" with modified parameters:
+$ limactl create --cpus=2 --memory=2
+
+To create an instance "default" with yq expressions:
+$ limactl create --set='.cpus = 2 | .memory = "2GiB"'
+
+To see the template list:
+$ limactl create --list-templates
+
+To create an instance "default" from a local file:
+$ limactl create --name=default /usr/local/share/lima/templates/fedora.yaml
+
+To create an instance "default" from a remote URL (use carefully, with a trustable source):
+$ limactl create --name=default https://raw.githubusercontent.com/lima-vm/lima/master/examples/alpine.yaml
+
+To create an instance "local" from a template passed to stdin (--name parameter is required):
+$ cat template.yaml | limactl create --name=local -
+`,
+		Short:             "Create an instance of Lima",
+		Args:              WrapArgsError(cobra.MaximumNArgs(1)),
+		ValidArgsFunction: createBashComplete,
+		RunE:              createAction,
+	}
+	registerCreateFlags(createCommand, "")
+	return createCommand
+}
+
 func newStartCommand() *cobra.Command {
 	var startCommand = &cobra.Command{
 		Use: "start NAME|FILE.yaml|URL",
@@ -34,39 +81,23 @@ func newStartCommand() *cobra.Command {
 To create an instance "default" (if not created yet) from the default Ubuntu template, and start it:
 $ limactl start
 
-To create an instance "default" from a template "docker":
+To create an instance "default" from a template "docker", and start it:
 $ limactl start --name=default template://docker
 
-To create an instance "default" with modified parameters:
-$ limactl start --set='.cpus = 2 | .memory = "2GiB"'
-
-To see the template list:
-$ limactl start --list-templates
-
-To create an instance "default" from a local file:
-$ limactl start --name=default /usr/local/share/lima/examples/fedora.yaml
-
-To create an instance "default" from a remote URL (use carefully, with a trustable source):
-$ limactl start --name=default https://raw.githubusercontent.com/lima-vm/lima/master/examples/alpine.yaml
-
-To create an instance "local" from a template passed to stdin (--name parameter is required):
-$ cat template.yaml | limactl start --name=local -
+'limactl start' also accepts the 'limactl create' flags such as '--set'.
+See the examples in 'limactl create --help'.
 `,
 		Short:             "Start an instance of Lima",
 		Args:              WrapArgsError(cobra.MaximumNArgs(1)),
 		ValidArgsFunction: startBashComplete,
 		RunE:              startAction,
 	}
-	// TODO: "survey" does not support using cygwin terminal on windows yet
-	startCommand.Flags().Bool("tty", isatty.IsTerminal(os.Stdout.Fd()), "enable TUI interactions such as opening an editor, defaults to true when stdout is a terminal")
-	startCommand.Flags().String("name", "", "override the instance name")
-	startCommand.Flags().String("set", "", "modify the template inplace, using yq syntax")
-	startCommand.Flags().Bool("list-templates", false, "list available templates and exit")
+	registerCreateFlags(startCommand, "[limactl create] ")
 	startCommand.Flags().Duration("timeout", start.DefaultWatchHostAgentEventsTimeout, "duration to wait for the instance to be running before timing out")
 	return startCommand
 }
 
-func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, error) {
+func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*store.Instance, error) {
 	var arg string // can be empty
 	if len(args) > 0 {
 		arg = args[0]
@@ -77,20 +108,19 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, e
 		err error
 	)
 
+	flags := cmd.Flags()
+
 	// Create an instance, with menu TUI when TTY is available
-	tty, err := cmd.Flags().GetBool("tty")
+	tty, err := flags.GetBool("tty")
 	if err != nil {
 		return nil, err
 	}
 
-	st.instName, err = cmd.Flags().GetString("name")
-	if err != nil {
-		return nil, err
-	}
-	st.yq, err = cmd.Flags().GetString("set")
+	st.instName, err = flags.GetString("name")
 	if err != nil {
 		return nil, err
 	}
+
 	const yBytesLimit = 4 * 1024 * 1024 // 4MiB
 
 	if ok, u := guessarg.SeemsTemplateURL(arg); ok {
@@ -187,9 +217,23 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, e
 		}
 		inst, err := store.Inspect(st.instName)
 		if err == nil {
+			if createOnly {
+				return nil, fmt.Errorf("Instance %q already exists", st.instName)
+			}
 			logrus.Infof("Using the existing instance %q", st.instName)
 			if arg == "" {
-				logrus.Infof("Hint: To create another instance, run the following command: limactl start --name=NAME template://default")
+				logrus.Infof("Hint: To create another instance, run the following command: limactl create --name=NAME template://default")
+			}
+			yqExprs, err := editflags.YQExpressions(flags, false)
+			if err != nil {
+				return nil, err
+			}
+			if len(yqExprs) > 0 {
+				yq := yqutil.Join(yqExprs)
+				inst, err = applyYQExpressionToExistingInstance(inst, yq)
+				if err != nil {
+					return nil, fmt.Errorf("failed to apply yq expression %q to instance %q: %w", yq, st.instName, err)
+				}
 			}
 			return inst, nil
 		}
@@ -198,7 +242,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, e
 		}
 		if arg != "" && arg != DefaultInstanceName {
 			logrus.Infof("Creating an instance %q from template://default (Not from template://%s)", st.instName, st.instName)
-			logrus.Warnf("This form is deprecated. Use `limactl start --name=%s template://default` instead", st.instName)
+			logrus.Warnf("This form is deprecated. Use `limactl create --name=%s template://default` instead", st.instName)
 		}
 		// Read the default template for creating a new instance
 		st.yBytes, err = templatestore.Read(templatestore.Default)
@@ -207,15 +251,20 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, e
 		}
 	}
 
+	yqExprs, err := editflags.YQExpressions(flags, true)
+	if err != nil {
+		return nil, err
+	}
+	yq := yqutil.Join(yqExprs)
 	if tty {
 		var err error
-		st, err = chooseNextCreatorState(st)
+		st, err = chooseNextCreatorState(st, yq)
 		if err != nil {
 			return nil, err
 		}
 	} else {
 		logrus.Info("Terminal is not available, proceeding without opening an editor")
-		if err := modifyInPlace(st); err != nil {
+		if err := modifyInPlace(st, yq); err != nil {
 			return nil, err
 		}
 	}
@@ -223,6 +272,27 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string) (*store.Instance, e
 	return createInstance(st, saveBrokenEditorBuffer)
 }
 
+func applyYQExpressionToExistingInstance(inst *store.Instance, yq string) (*store.Instance, error) {
+	if strings.TrimSpace(yq) == "" {
+		return inst, nil
+	}
+	filePath := filepath.Join(inst.Dir, filenames.LimaYAML)
+	yContent, err := os.ReadFile(filePath)
+	if err != nil {
+		return nil, err
+	}
+	logrus.Debugf("Applying yq expression %q to an existing instance %q", yq, inst.Name)
+	yBytes, err := yqutil.EvaluateExpression(yq, yContent)
+	if err != nil {
+		return nil, err
+	}
+	if err := os.WriteFile(filePath, yBytes, 0644); err != nil {
+		return nil, err
+	}
+	// Reload
+	return store.Inspect(inst.Name)
+}
+
 func createInstance(st *creatorState, saveBrokenEditorBuffer bool) (*store.Instance, error) {
 	if st.instName == "" {
 		return nil, errors.New("got empty st.instName")
@@ -273,15 +343,10 @@ func createInstance(st *creatorState, saveBrokenEditorBuffer bool) (*store.Insta
 type creatorState struct {
 	instName string // instance name
 	yBytes   []byte // yaml bytes
-	yq       string // yq expression
 }
 
-func modifyInPlace(st *creatorState) error {
-	if st.yq == "" {
-		return nil
-	}
-	logrus.Warn("`--set` is experimental")
-	out, err := yqutil.EvaluateExpression(st.yq, st.yBytes)
+func modifyInPlace(st *creatorState, yq string) error {
+	out, err := yqutil.EvaluateExpression(yq, st.yBytes)
 	if err != nil {
 		return err
 	}
@@ -289,9 +354,9 @@ func modifyInPlace(st *creatorState) error {
 	return nil
 }
 
-func chooseNextCreatorState(st *creatorState) (*creatorState, error) {
+func chooseNextCreatorState(st *creatorState, yq string) (*creatorState, error) {
 	for {
-		if err := modifyInPlace(st); err != nil {
+		if err := modifyInPlace(st, yq); err != nil {
 			logrus.WithError(err).Warn("Failed to evaluate yq expression")
 			return st, err
 		}
@@ -301,7 +366,7 @@ func chooseNextCreatorState(st *creatorState) (*creatorState, error) {
 			Options: []string{
 				"Proceed with the current configuration",
 				"Open an editor to review or modify the current configuration",
-				"Choose another example (docker, podman, archlinux, fedora, ...)",
+				"Choose another template (docker, podman, archlinux, fedora, ...)",
 				"Exit",
 			},
 		}
@@ -321,7 +386,7 @@ func chooseNextCreatorState(st *creatorState) (*creatorState, error) {
 			hdr += "\n"
 			hdr += editutil.GenerateEditorWarningHeader()
 			var err error
-			st.yBytes, err = editutil.OpenEditor(st.instName, st.yBytes, hdr)
+			st.yBytes, err = editutil.OpenEditor(st.yBytes, hdr)
 			if err != nil {
 				return st, err
 			}
@@ -331,26 +396,26 @@ func chooseNextCreatorState(st *creatorState) (*creatorState, error) {
 				return st, errors.New("should not reach here")
 			}
 			return st, nil
-		case prompt.Options[2]: // "Choose another example..."
-			examples, err := templatestore.Templates()
+		case prompt.Options[2]: // "Choose another template..."
+			templates, err := templatestore.Templates()
 			if err != nil {
 				return st, err
 			}
 			var ansEx int
 			promptEx := &survey.Select{
-				Message: "Choose an example",
-				Options: make([]string, len(examples)),
+				Message: "Choose a template",
+				Options: make([]string, len(templates)),
 			}
-			for i := range examples {
-				promptEx.Options[i] = examples[i].Name
+			for i := range templates {
+				promptEx.Options[i] = templates[i].Name
 			}
 			if err := survey.AskOne(promptEx, &ansEx); err != nil {
 				return st, err
 			}
-			if ansEx > len(examples)-1 {
-				return st, fmt.Errorf("invalid answer %d for %d entries", ansEx, len(examples))
+			if ansEx > len(templates)-1 {
+				return st, fmt.Errorf("invalid answer %d for %d entries", ansEx, len(templates))
 			}
-			yamlPath := examples[ansEx].Location
+			yamlPath := templates[ansEx].Location
 			st.instName, err = guessarg.InstNameFromYAMLPath(yamlPath)
 			if err != nil {
 				return nil, err
@@ -369,20 +434,49 @@ func chooseNextCreatorState(st *creatorState) (*creatorState, error) {
 	}
 }
 
-func startAction(cmd *cobra.Command, args []string) error {
+// createStartActionCommon is shared by createAction and startAction.
+func createStartActionCommon(cmd *cobra.Command, _ []string) (exit bool, err error) {
 	if listTemplates, err := cmd.Flags().GetBool("list-templates"); err != nil {
-		return err
+		return true, err
 	} else if listTemplates {
 		if templates, err := templatestore.Templates(); err == nil {
 			w := cmd.OutOrStdout()
 			for _, f := range templates {
 				fmt.Fprintln(w, f.Name)
 			}
-			return nil
+			return true, nil
 		}
 	}
+	return false, nil
+}
 
-	inst, err := loadOrCreateInstance(cmd, args)
+func createAction(cmd *cobra.Command, args []string) error {
+	if exit, err := createStartActionCommon(cmd, args); err != nil {
+		return err
+	} else if exit {
+		return nil
+	}
+	inst, err := loadOrCreateInstance(cmd, args, true)
+	if err != nil {
+		return err
+	}
+	if len(inst.Errors) > 0 {
+		return fmt.Errorf("errors inspecting instance: %+v", inst.Errors)
+	}
+	if _, err = start.Prepare(cmd.Context(), inst); err != nil {
+		return err
+	}
+	logrus.Infof("Run `limactl start %s` to start the instance.", inst.Name)
+	return nil
+}
+
+func startAction(cmd *cobra.Command, args []string) error {
+	if exit, err := createStartActionCommon(cmd, args); err != nil {
+		return err
+	} else if exit {
+		return nil
+	}
+	inst, err := loadOrCreateInstance(cmd, args, false)
 	if err != nil {
 		return err
 	}
@@ -417,12 +511,12 @@ func startAction(cmd *cobra.Command, args []string) error {
 	return start.Start(ctx, inst)
 }
 
-func startBashComplete(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-	comp, _ := bashCompleteInstanceNames(cmd)
-	if templates, err := templatestore.Templates(); err == nil {
-		for _, f := range templates {
-			comp = append(comp, "template://"+f.Name)
-		}
-	}
-	return comp, cobra.ShellCompDirectiveDefault
+func createBashComplete(cmd *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	return bashCompleteTemplateNames(cmd)
+}
+
+func startBashComplete(cmd *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
+	compInst, _ := bashCompleteInstanceNames(cmd)
+	compTmpl, _ := bashCompleteTemplateNames(cmd)
+	return append(compInst, compTmpl...), cobra.ShellCompDirectiveDefault
 }
