@@ -113,7 +113,8 @@ func stopInstanceForcibly(inst *store.Instance) {
 		logrus.Infof("The %s driver process seems already stopped", inst.VMType)
 	}
 
-	for _, diskName := range inst.AdditionalDisks {
+	for _, d := range inst.AdditionalDisks {
+		diskName := d.Name
 		disk, err := store.InspectDisk(diskName)
 		if err != nil {
 			logrus.Warnf("Disk %q does not exist", diskName)
@@ -133,7 +134,8 @@ func stopInstanceForcibly(inst *store.Instance) {
 		logrus.Info("The host agent process seems already stopped")
 	}
 
-	logrus.Infof("Removing *.pid *.sock under %q", inst.Dir)
+	suffixesToBeRemoved := []string{".pid", ".sock", ".tmp"}
+	logrus.Infof("Removing %s under %q", inst.Dir, strings.ReplaceAll(strings.Join(suffixesToBeRemoved, " "), ".", "*."))
 	fi, err := os.ReadDir(inst.Dir)
 	if err != nil {
 		logrus.Error(err)
@@ -141,15 +143,17 @@ func stopInstanceForcibly(inst *store.Instance) {
 	}
 	for _, f := range fi {
 		path := filepath.Join(inst.Dir, f.Name())
-		if strings.HasSuffix(path, ".pid") || strings.HasSuffix(path, ".sock") {
-			logrus.Infof("Removing %q", path)
-			if err := os.Remove(path); err != nil {
-				logrus.Error(err)
+		for _, suffix := range suffixesToBeRemoved {
+			if strings.HasSuffix(path, suffix) {
+				logrus.Infof("Removing %q", path)
+				if err := os.Remove(path); err != nil {
+					logrus.Error(err)
+				}
 			}
 		}
 	}
 }
 
-func stopBashComplete(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+func stopBashComplete(cmd *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
 	return bashCompleteInstanceNames(cmd)
 }
