@@ -118,16 +118,17 @@ func GenerateISO9660(instDir, name string, y *limayaml.LimaYAML, udpDNSLocalPort
 		return err
 	}
 	args := TemplateArgs{
-		Name:           name,
-		User:           u.Username,
-		UID:            uid,
-		Containerd:     Containerd{System: *y.Containerd.System, User: *y.Containerd.User},
-		SlirpNICName:   networks.SlirpNICName,
-		SlirpGateway:   networks.SlirpGateway,
-		SlirpDNS:       networks.SlirpDNS,
-		SlirpIPAddress: networks.SlirpIPAddress,
-		RosettaEnabled: *y.Rosetta.Enabled,
-		RosettaBinFmt:  *y.Rosetta.BinFmt,
+		Name:               name,
+		User:               u.Username,
+		UID:                uid,
+		GuestInstallPrefix: *y.GuestInstallPrefix,
+		Containerd:         Containerd{System: *y.Containerd.System, User: *y.Containerd.User},
+		SlirpNICName:       networks.SlirpNICName,
+		SlirpGateway:       networks.SlirpGateway,
+		SlirpDNS:           networks.SlirpDNS,
+		SlirpIPAddress:     networks.SlirpIPAddress,
+		RosettaEnabled:     *y.Rosetta.Enabled,
+		RosettaBinFmt:      *y.Rosetta.BinFmt,
 	}
 
 	// change instance id on every boot so network config will be processed again
@@ -202,10 +203,21 @@ func GenerateISO9660(instDir, name string, y *limayaml.LimaYAML, udpDNSLocalPort
 		args.MountType = "virtiofs"
 	}
 
-	for i, disk := range y.AdditionalDisks {
+	for i, d := range y.AdditionalDisks {
+		format := true
+		if d.Format != nil {
+			format = *d.Format
+		}
+		fstype := ""
+		if d.FSType != nil {
+			fstype = *d.FSType
+		}
 		args.Disks = append(args.Disks, Disk{
-			Name:   disk,
+			Name:   d.Name,
 			Device: diskDeviceNameFromOrder(i),
+			Format: format,
+			FSType: fstype,
+			FSArgs: d.FSArgs,
 		})
 	}
 
