@@ -335,12 +335,24 @@ func (a *HostAgent) Run(ctx context.Context) error {
 		logrus.Infof("VNC Password: `%s`", vncpwdfile)
 	}
 
+	if a.driver.CanRunGUI() {
+		go func() {
+			err = a.startRoutinesAndWait(ctx, errCh)
+			if err != nil {
+				logrus.Error(err)
+			}
+		}()
+		return a.driver.RunGUI()
+	}
+	return a.startRoutinesAndWait(ctx, errCh)
+}
+
+func (a *HostAgent) startRoutinesAndWait(ctx context.Context, errCh chan error) error {
 	stBase := events.Status{
 		SSHLocalPort: a.sshLocalPort,
 	}
 	stBooting := stBase
 	a.emitEvent(ctx, events.Event{Status: stBooting})
-
 	ctxHA, cancelHA := context.WithCancel(ctx)
 	go func() {
 		stRunning := stBase
@@ -351,7 +363,6 @@ func (a *HostAgent) Run(ctx context.Context) error {
 		stRunning.Running = true
 		a.emitEvent(ctx, events.Event{Status: stRunning})
 	}()
-
 	for {
 		select {
 		case driverErr := <-errCh:
@@ -390,7 +401,7 @@ func (a *HostAgent) startHostAgentRoutines(ctx context.Context) error {
 		return nil
 	})
 	var mErr error
-	if err := a.waitForRequirements(ctx, "essential", a.essentialRequirements()); err != nil {
+	if err := a.waitForRequirements("essential", a.essentialRequirements()); err != nil {
 		mErr = multierror.Append(mErr, err)
 	}
 	if *a.y.SSH.ForwardAgent {
@@ -407,7 +418,7 @@ sudo chown -R "${USER}" /run/host-services`
 		}
 	}
 	if *a.y.MountType == limayaml.REVSSHFS {
-		mounts, err := a.setupMounts(ctx)
+		mounts, err := a.setupMounts()
 		if err != nil {
 			mErr = multierror.Append(mErr, err)
 		}
@@ -425,7 +436,7 @@ sudo chown -R "${USER}" /run/host-services`
 		a.onClose = append(a.onClose, func() error {
 			var unlockMErr error
 			for _, d := range a.y.AdditionalDisks {
-				disk, inspectErr := store.InspectDisk(d)
+				disk, inspectErr := store.InspectDisk(d.Name)
 				if inspectErr != nil {
 					unlockMErr = multierror.Append(unlockMErr, inspectErr)
 					continue
@@ -439,10 +450,10 @@ sudo chown -R "${USER}" /run/host-services`
 		})
 	}
 	go a.watchGuestAgentEvents(ctx)
-	if err := a.waitForRequirements(ctx, "optional", a.optionalRequirements()); err != nil {
+	if err := a.waitForRequirements("optional", a.optionalRequirements()); err != nil {
 		mErr = multierror.Append(mErr, err)
 	}
-	if err := a.waitForRequirements(ctx, "final", a.finalRequirements()); err != nil {
+	if err := a.waitForRequirements("final", a.finalRequirements()); err != nil {
 		mErr = multierror.Append(mErr, err)
 	}
 	// Copy all config files _after_ the requirements are done
