@@ -11,7 +11,9 @@ import (
 	"strconv"
 	"text/template"
 
+	"github.com/docker/go-units"
 	"github.com/lima-vm/lima/pkg/networks"
+	"github.com/pbnjay/memory"
 
 	"github.com/lima-vm/lima/pkg/guestagent/api"
 	"github.com/lima-vm/lima/pkg/osutil"
@@ -19,6 +21,8 @@ import (
 	"github.com/lima-vm/lima/pkg/store/filenames"
 	"github.com/sirupsen/logrus"
 	"github.com/xorcare/pointer"
+
+	"golang.org/x/sys/cpu"
 )
 
 const (
@@ -29,10 +33,12 @@ const (
 	Default9pMsize           string = "128KiB"
 	Default9pCacheForRO      string = "fscache"
 	Default9pCacheForRW      string = "mmap"
+
+	DefaultVirtiofsQueueSize int = 1024
 )
 
 func defaultContainerdArchives() []File {
-	const nerdctlVersion = "1.4.0"
+	const nerdctlVersion = "1.5.0"
 	location := func(goarch string) string {
 		return "https://github.com/containerd/nerdctl/releases/download/v" + nerdctlVersion + "/nerdctl-full-" + nerdctlVersion + "-linux-" + goarch + ".tar.gz"
 	}
@@ -40,13 +46,14 @@ func defaultContainerdArchives() []File {
 		{
 			Location: location("amd64"),
 			Arch:     X8664,
-			Digest:   "sha256:3bb5f5358ee2c3bd9097e0bf37649c1775e2449094e75acf629f129ec2c7915f",
+			Digest:   "sha256:3f8c494e3c6a265fe2a3c41ef9d6bc859eeeb22095b3353d3558d8120833a23a",
 		},
 		{
 			Location: location("arm64"),
 			Arch:     AARCH64,
-			Digest:   "sha256:589dabd962d936b29fd377dcddbb49c07d1c4c27dd4b402bc4b6b20287fe9c37",
+			Digest:   "sha256:32a2537e0a80e1493b5934ca56c3e237466606a1b720aef23b9c0a7fc3303bdb",
 		},
+		// No arm-v7
 		// No riscv64
 	}
 }
@@ -76,6 +83,35 @@ func MACAddress(uniqueID string) string {
 	return hw.String()
 }
 
+func defaultCPUs() int {
+	const x = 4
+	if hostCPUs := runtime.NumCPU(); hostCPUs < x {
+		return hostCPUs
+	}
+	return x
+}
+
+func defaultMemory() uint64 {
+	const x uint64 = 4 * 1024 * 1024 * 1024
+	if halfOfHostMemory := memory.TotalMemory() / 2; halfOfHostMemory < x {
+		return halfOfHostMemory
+	}
+	return x
+}
+
+func defaultMemoryAsString() string {
+	return units.BytesSize(float64(defaultMemory()))
+}
+
+func defaultDiskSizeAsString() string {
+	// currently just hardcoded
+	return "100GiB"
+}
+
+func defaultGuestInstallPrefix() string {
+	return "/usr/local"
+}
+
 // FillDefault updates undefined fields in y with defaults from d (or built-in default), and overwrites with values from o.
 // Both d and o may be empty.
 //
@@ -122,6 +158,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 
 	cpuType := map[Arch]string{
 		AARCH64: "cortex-a72",
+		ARMV7L:  "cortex-a7",
 		// Since https://github.com/lima-vm/lima/pull/494, we use qemu64 cpu for better emulation of x86_64.
 		X8664:   "qemu64",
 		RISCV64: "rv64", // FIXME: what is the right choice for riscv64?
@@ -174,7 +211,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.CPUs = o.CPUs
 	}
 	if y.CPUs == nil || *y.CPUs == 0 {
-		y.CPUs = pointer.Int(4)
+		y.CPUs = pointer.Int(defaultCPUs())
 	}
 
 	if y.Memory == nil {
@@ -184,7 +221,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.Memory = o.Memory
 	}
 	if y.Memory == nil || *y.Memory == "" {
-		y.Memory = pointer.String("4GiB")
+		y.Memory = pointer.String(defaultMemoryAsString())
 	}
 
 	if y.Disk == nil {
@@ -194,7 +231,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.Disk = o.Disk
 	}
 	if y.Disk == nil || *y.Disk == "" {
-		y.Disk = pointer.String("100GiB")
+		y.Disk = pointer.String(defaultDiskSizeAsString())
 	}
 
 	y.AdditionalDisks = append(append(o.AdditionalDisks, y.AdditionalDisks...), d.AdditionalDisks...)
@@ -215,7 +252,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 	if o.Video.Display != nil {
 		y.Video.Display = o.Video.Display
 	}
-	if (y.Video.Display == nil || *y.Video.Display == "") && *y.VMType == QEMU {
+	if y.Video.Display == nil || *y.Video.Display == "" {
 		y.Video.Display = pointer.String("none")
 	}
 
@@ -313,6 +350,16 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		}
 	}
 
+	if y.GuestInstallPrefix == nil {
+		y.GuestInstallPrefix = d.GuestInstallPrefix
+	}
+	if o.GuestInstallPrefix != nil {
+		y.GuestInstallPrefix = o.GuestInstallPrefix
+	}
+	if y.GuestInstallPrefix == nil {
+		y.GuestInstallPrefix = pointer.String(defaultGuestInstallPrefix())
+	}
+
 	if y.Containerd.System == nil {
 		y.Containerd.System = d.Containerd.System
 	}
@@ -456,6 +503,21 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		}
 	}
 
+	// MountType has to be resolved before resolving Mounts
+	if y.MountType == nil {
+		y.MountType = d.MountType
+	}
+	if o.MountType != nil {
+		y.MountType = o.MountType
+	}
+	if y.MountType == nil || *y.MountType == "" {
+		if *y.VMType == VZ {
+			y.MountType = pointer.String(VIRTIOFS)
+		} else {
+			y.MountType = pointer.String(REVSSHFS)
+		}
+	}
+
 	// Combine all mounts; highest priority entry determines writable status.
 	// Only works for exact matches; does not normalize case or resolve symlinks.
 	mounts := make([]Mount, 0, len(d.Mounts)+len(y.Mounts)+len(o.Mounts))
@@ -483,6 +545,9 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 			if mount.NineP.Cache != nil {
 				mounts[i].NineP.Cache = mount.NineP.Cache
 			}
+			if mount.Virtiofs.QueueSize != nil {
+				mounts[i].Virtiofs.QueueSize = mount.Virtiofs.QueueSize
+			}
 			if mount.Writable != nil {
 				mounts[i].Writable = mount.Writable
 			}
@@ -516,6 +581,9 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		if mount.NineP.Msize == nil {
 			mounts[i].NineP.Msize = pointer.String(Default9pMsize)
 		}
+		if mount.Virtiofs.QueueSize == nil && *y.VMType == QEMU && *y.MountType == VIRTIOFS {
+			mounts[i].Virtiofs.QueueSize = pointer.Int(DefaultVirtiofsQueueSize)
+		}
 		if mount.Writable == nil {
 			mount.Writable = pointer.Bool(false)
 		}
@@ -531,16 +599,6 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		}
 	}
 
-	if y.MountType == nil {
-		y.MountType = d.MountType
-	}
-	if o.MountType != nil {
-		y.MountType = o.MountType
-	}
-	if y.MountType == nil || *y.MountType == "" {
-		y.MountType = pointer.String(REVSSHFS)
-	}
-
 	// Note: DNS lists are not combined; highest priority setting is picked
 	if len(y.DNS) == 0 {
 		y.DNS = d.DNS
@@ -577,13 +635,17 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 	caCerts := unique(append(append(d.CACertificates.Certs, y.CACertificates.Certs...), o.CACertificates.Certs...))
 	y.CACertificates.Certs = caCerts
 
-	if y.Rosetta.Enabled == nil {
-		y.Rosetta.Enabled = d.Rosetta.Enabled
-	}
-	if o.Rosetta.Enabled != nil {
-		y.Rosetta.Enabled = o.Rosetta.Enabled
-	}
-	if y.Rosetta.Enabled == nil {
+	if runtime.GOOS == "darwin" && IsNativeArch(AARCH64) {
+		if y.Rosetta.Enabled == nil {
+			y.Rosetta.Enabled = d.Rosetta.Enabled
+		}
+		if o.Rosetta.Enabled != nil {
+			y.Rosetta.Enabled = o.Rosetta.Enabled
+		}
+		if y.Rosetta.Enabled == nil {
+			y.Rosetta.Enabled = pointer.Bool(false)
+		}
+	} else {
 		y.Rosetta.Enabled = pointer.Bool(false)
 	}
 
@@ -706,12 +768,35 @@ func FillCopyToHostDefaults(rule *CopyToHost, instDir string) {
 	}
 }
 
+func goarm() int {
+	if runtime.GOOS != "linux" {
+		return 0
+	}
+	if runtime.GOARCH != "arm" {
+		return 0
+	}
+	if cpu.ARM.HasVFPv3 {
+		return 7
+	}
+	if cpu.ARM.HasVFP {
+		return 6
+	}
+	return 5 // default
+}
+
 func NewArch(arch string) Arch {
 	switch arch {
 	case "amd64":
 		return X8664
 	case "arm64":
 		return AARCH64
+	case "arm":
+		arm := goarm()
+		if arm == 7 {
+			return ARMV7L
+		}
+		logrus.Warnf("Unknown arm: %d", arm)
+		return arch
 	case "riscv64":
 		return RISCV64
 	default:
@@ -775,8 +860,9 @@ func HasMaxCPU() bool {
 func IsNativeArch(arch Arch) bool {
 	nativeX8664 := arch == X8664 && runtime.GOARCH == "amd64"
 	nativeAARCH64 := arch == AARCH64 && runtime.GOARCH == "arm64"
+	nativeARMV7L := arch == ARMV7L && runtime.GOARCH == "arm" && goarm() == 7
 	nativeRISCV64 := arch == RISCV64 && runtime.GOARCH == "riscv64"
-	return nativeX8664 || nativeAARCH64 || nativeRISCV64
+	return nativeX8664 || nativeAARCH64 || nativeARMV7L || nativeRISCV64
 }
 
 func unique(s []string) []string {
