@@ -32,6 +32,14 @@ func TestFillDefault(t *testing.T) {
 		arch = X8664
 	case "arm64":
 		arch = AARCH64
+	case "arm":
+		if runtime.GOOS != "linux" {
+			t.Skipf("unsupported GOOS: %s", runtime.GOOS)
+		}
+		if arm := goarm(); arm < 7 {
+			t.Skipf("unsupported GOARM: %d", arm)
+		}
+		arch = ARMV7L
 	case "riscv64":
 		arch = RISCV64
 	default:
@@ -56,12 +64,14 @@ func TestFillDefault(t *testing.T) {
 		Arch:   pointer.String(arch),
 		CPUType: map[Arch]string{
 			AARCH64: "cortex-a72",
+			ARMV7L:  "cortex-a7",
 			X8664:   "qemu64",
 			RISCV64: "rv64",
 		},
-		CPUs:   pointer.Int(4),
-		Memory: pointer.String("4GiB"),
-		Disk:   pointer.String("100GiB"),
+		CPUs:               pointer.Int(defaultCPUs()),
+		Memory:             pointer.String(defaultMemoryAsString()),
+		Disk:               pointer.String(defaultDiskSizeAsString()),
+		GuestInstallPrefix: pointer.String(defaultGuestInstallPrefix()),
 		Containerd: Containerd{
 			System:   pointer.Bool(false),
 			User:     pointer.Bool(true),
@@ -186,6 +196,7 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.ProtocolVersion = pointer.String(Default9pProtocolVersion)
 	expect.Mounts[0].NineP.Msize = pointer.String(Default9pMsize)
 	expect.Mounts[0].NineP.Cache = pointer.String(Default9pCacheForRO)
+	expect.Mounts[0].Virtiofs.QueueSize = pointer.Int(DefaultVirtiofsQueueSize)
 	// Only missing Mounts field is Writable, and the default value is also the null value: false
 
 	expect.MountType = pointer.String(NINEP)
@@ -258,6 +269,7 @@ func TestFillDefault(t *testing.T) {
 		Arch:   pointer.String("unknown"),
 		CPUType: map[Arch]string{
 			AARCH64: "arm64",
+			ARMV7L:  "armhf",
 			X8664:   "amd64",
 			RISCV64: "riscv64",
 		},
@@ -265,8 +277,9 @@ func TestFillDefault(t *testing.T) {
 		Memory: pointer.String("5GiB"),
 		Disk:   pointer.String("105GiB"),
 		AdditionalDisks: []Disk{
-			"data",
+			{Name: "data"},
 		},
+		GuestInstallPrefix: pointer.String("/opt"),
 		Containerd: Containerd{
 			System: pointer.Bool(true),
 			User:   pointer.Bool(false),
@@ -369,18 +382,26 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.ProtocolVersion = pointer.String(Default9pProtocolVersion)
 	expect.Mounts[0].NineP.Msize = pointer.String(Default9pMsize)
 	expect.Mounts[0].NineP.Cache = pointer.String(Default9pCacheForRO)
+	expect.Mounts[0].Virtiofs.QueueSize = pointer.Int(DefaultVirtiofsQueueSize)
 	expect.HostResolver.Hosts = map[string]string{
 		"default": d.HostResolver.Hosts["default"],
 	}
-	expect.MountType = pointer.String(REVSSHFS)
+	expect.MountType = pointer.String(VIRTIOFS)
 	expect.CACertificates.RemoveDefaults = pointer.Bool(true)
 	expect.CACertificates.Certs = []string{
 		"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
 	}
 
-	expect.Rosetta = Rosetta{
-		Enabled: pointer.Bool(true),
-		BinFmt:  pointer.Bool(true),
+	if runtime.GOOS == "darwin" && IsNativeArch(AARCH64) {
+		expect.Rosetta = Rosetta{
+			Enabled: pointer.Bool(true),
+			BinFmt:  pointer.Bool(true),
+		}
+	} else {
+		expect.Rosetta = Rosetta{
+			Enabled: pointer.Bool(false),
+			BinFmt:  pointer.Bool(true),
+		}
 	}
 
 	y = LimaYAML{}
@@ -392,7 +413,7 @@ func TestFillDefault(t *testing.T) {
 
 	y = filledDefaults
 	y.DNS = []net.IP{net.ParseIP("8.8.8.8")}
-	y.AdditionalDisks = []Disk{"overridden"}
+	y.AdditionalDisks = []Disk{{Name: "overridden"}}
 
 	expect = y
 
@@ -425,6 +446,7 @@ func TestFillDefault(t *testing.T) {
 		Arch:   pointer.String(arch),
 		CPUType: map[Arch]string{
 			AARCH64: "uber-arm",
+			ARMV7L:  "armv8",
 			X8664:   "pentium",
 			RISCV64: "sifive-u54",
 		},
@@ -432,8 +454,9 @@ func TestFillDefault(t *testing.T) {
 		Memory: pointer.String("7GiB"),
 		Disk:   pointer.String("117GiB"),
 		AdditionalDisks: []Disk{
-			"test",
+			{Name: "test"},
 		},
+		GuestInstallPrefix: pointer.String("/usr"),
 		Containerd: Containerd{
 			System: pointer.Bool(true),
 			User:   pointer.Bool(false),
@@ -487,6 +510,9 @@ func TestFillDefault(t *testing.T) {
 					Msize:           pointer.String("8KiB"),
 					Cache:           pointer.String("none"),
 				},
+				Virtiofs: Virtiofs{
+					QueueSize: pointer.Int(2048),
+				},
 			},
 		},
 		Provision: []Provision{
@@ -562,6 +588,7 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.ProtocolVersion = pointer.String("9p2000")
 	expect.Mounts[0].NineP.Msize = pointer.String("8KiB")
 	expect.Mounts[0].NineP.Cache = pointer.String("none")
+	expect.Mounts[0].Virtiofs.QueueSize = pointer.Int(2048)
 
 	expect.MountType = pointer.String(NINEP)
 
