@@ -7,27 +7,28 @@ import (
 )
 
 type LimaYAML struct {
-	VMType          *VMType         `yaml:"vmType,omitempty" json:"vmType,omitempty"`
-	Arch            *Arch           `yaml:"arch,omitempty" json:"arch,omitempty"`
-	Images          []Image         `yaml:"images" json:"images"` // REQUIRED
-	CPUType         map[Arch]string `yaml:"cpuType,omitempty" json:"cpuType,omitempty"`
-	CPUs            *int            `yaml:"cpus,omitempty" json:"cpus,omitempty"`
-	Memory          *string         `yaml:"memory,omitempty" json:"memory,omitempty"` // go-units.RAMInBytes
-	Disk            *string         `yaml:"disk,omitempty" json:"disk,omitempty"`     // go-units.RAMInBytes
-	AdditionalDisks []Disk          `yaml:"additionalDisks,omitempty" json:"additionalDisks,omitempty"`
-	Mounts          []Mount         `yaml:"mounts,omitempty" json:"mounts,omitempty"`
-	MountType       *MountType      `yaml:"mountType,omitempty" json:"mountType,omitempty"`
-	SSH             SSH             `yaml:"ssh,omitempty" json:"ssh,omitempty"` // REQUIRED (FIXME)
-	Firmware        Firmware        `yaml:"firmware,omitempty" json:"firmware,omitempty"`
-	Audio           Audio           `yaml:"audio,omitempty" json:"audio,omitempty"`
-	Video           Video           `yaml:"video,omitempty" json:"video,omitempty"`
-	Provision       []Provision     `yaml:"provision,omitempty" json:"provision,omitempty"`
-	Containerd      Containerd      `yaml:"containerd,omitempty" json:"containerd,omitempty"`
-	Probes          []Probe         `yaml:"probes,omitempty" json:"probes,omitempty"`
-	PortForwards    []PortForward   `yaml:"portForwards,omitempty" json:"portForwards,omitempty"`
-	CopyToHost      []CopyToHost    `yaml:"copyToHost,omitempty" json:"copyToHost,omitempty"`
-	Message         string          `yaml:"message,omitempty" json:"message,omitempty"`
-	Networks        []Network       `yaml:"networks,omitempty" json:"networks,omitempty"`
+	VMType             *VMType         `yaml:"vmType,omitempty" json:"vmType,omitempty"`
+	Arch               *Arch           `yaml:"arch,omitempty" json:"arch,omitempty"`
+	Images             []Image         `yaml:"images" json:"images"` // REQUIRED
+	CPUType            map[Arch]string `yaml:"cpuType,omitempty" json:"cpuType,omitempty"`
+	CPUs               *int            `yaml:"cpus,omitempty" json:"cpus,omitempty"`
+	Memory             *string         `yaml:"memory,omitempty" json:"memory,omitempty"` // go-units.RAMInBytes
+	Disk               *string         `yaml:"disk,omitempty" json:"disk,omitempty"`     // go-units.RAMInBytes
+	AdditionalDisks    []Disk          `yaml:"additionalDisks,omitempty" json:"additionalDisks,omitempty"`
+	Mounts             []Mount         `yaml:"mounts,omitempty" json:"mounts,omitempty"`
+	MountType          *MountType      `yaml:"mountType,omitempty" json:"mountType,omitempty"`
+	SSH                SSH             `yaml:"ssh,omitempty" json:"ssh,omitempty"` // REQUIRED (FIXME)
+	Firmware           Firmware        `yaml:"firmware,omitempty" json:"firmware,omitempty"`
+	Audio              Audio           `yaml:"audio,omitempty" json:"audio,omitempty"`
+	Video              Video           `yaml:"video,omitempty" json:"video,omitempty"`
+	Provision          []Provision     `yaml:"provision,omitempty" json:"provision,omitempty"`
+	Containerd         Containerd      `yaml:"containerd,omitempty" json:"containerd,omitempty"`
+	GuestInstallPrefix *string         `yaml:"guestInstallPrefix,omitempty" json:"guestInstallPrefix,omitempty"`
+	Probes             []Probe         `yaml:"probes,omitempty" json:"probes,omitempty"`
+	PortForwards       []PortForward   `yaml:"portForwards,omitempty" json:"portForwards,omitempty"`
+	CopyToHost         []CopyToHost    `yaml:"copyToHost,omitempty" json:"copyToHost,omitempty"`
+	Message            string          `yaml:"message,omitempty" json:"message,omitempty"`
+	Networks           []Network       `yaml:"networks,omitempty" json:"networks,omitempty"`
 	// `network` was deprecated in Lima v0.7.0, removed in Lima v0.14.0. Use `networks` instead.
 	Env          map[string]string `yaml:"env,omitempty" json:"env,omitempty"`
 	DNS          []net.IP          `yaml:"dns,omitempty" json:"dns,omitempty"`
@@ -45,6 +46,7 @@ type VMType = string
 const (
 	X8664   Arch = "x86_64"
 	AARCH64 Arch = "aarch64"
+	ARMV7L  Arch = "armv7l"
 	RISCV64 Arch = "riscv64"
 
 	REVSSHFS MountType = "reverse-sshfs"
@@ -77,14 +79,20 @@ type Image struct {
 	Initrd *File   `yaml:"initrd,omitempty" json:"initrd,omitempty"`
 }
 
-type Disk = string
+type Disk struct {
+	Name   string   `yaml:"name" json:"name"` // REQUIRED
+	Format *bool    `yaml:"format,omitempty" json:"format,omitempty"`
+	FSType *string  `yaml:"fsType,omitempty" json:"fsType,omitempty"`
+	FSArgs []string `yaml:"fsArgs,omitempty" json:"fsArgs,omitempty"`
+}
 
 type Mount struct {
-	Location   string `yaml:"location" json:"location"` // REQUIRED
-	MountPoint string `yaml:"mountPoint,omitempty" json:"mountPoint,omitempty"`
-	Writable   *bool  `yaml:"writable,omitempty" json:"writable,omitempty"`
-	SSHFS      SSHFS  `yaml:"sshfs,omitempty" json:"sshfs,omitempty"`
-	NineP      NineP  `yaml:"9p,omitempty" json:"9p,omitempty"`
+	Location   string   `yaml:"location" json:"location"` // REQUIRED
+	MountPoint string   `yaml:"mountPoint,omitempty" json:"mountPoint,omitempty"`
+	Writable   *bool    `yaml:"writable,omitempty" json:"writable,omitempty"`
+	SSHFS      SSHFS    `yaml:"sshfs,omitempty" json:"sshfs,omitempty"`
+	NineP      NineP    `yaml:"9p,omitempty" json:"9p,omitempty"`
+	Virtiofs   Virtiofs `yaml:"virtiofs,omitempty" json:"virtiofs,omitempty"`
 }
 
 type SFTPDriver = string
@@ -107,6 +115,10 @@ type NineP struct {
 	Cache           *string `yaml:"cache,omitempty" json:"cache,omitempty"`
 }
 
+type Virtiofs struct {
+	QueueSize *int `yaml:"queueSize,omitempty" json:"queueSize,omitempty"`
+}
+
 type SSH struct {
 	LocalPort *int `yaml:"localPort,omitempty" json:"localPort,omitempty"`
 
