@@ -26,9 +26,9 @@ func validateFileObject(f File, fieldName string) error {
 		// f.Location does NOT need to be accessible, so we do NOT check os.Stat(f.Location)
 	}
 	switch f.Arch {
-	case X8664, AARCH64, RISCV64:
+	case X8664, AARCH64, ARMV7L, RISCV64:
 	default:
-		return fmt.Errorf("field `arch` must be %q, %q, or %q; got %q", X8664, AARCH64, RISCV64, f.Arch)
+		return fmt.Errorf("field `arch` must be %q, %q, %q, or %q; got %q", X8664, AARCH64, ARMV7L, RISCV64, f.Arch)
 	}
 	if f.Digest != "" {
 		if !f.Digest.Algorithm().Available() {
@@ -43,9 +43,9 @@ func validateFileObject(f File, fieldName string) error {
 
 func Validate(y LimaYAML, warn bool) error {
 	switch *y.Arch {
-	case X8664, AARCH64, RISCV64:
+	case X8664, AARCH64, ARMV7L, RISCV64:
 	default:
-		return fmt.Errorf("field `arch` must be %q, %q, or %q; got %q", X8664, AARCH64, RISCV64, *y.Arch)
+		return fmt.Errorf("field `arch` must be %q, %q, %q or %q; got %q", X8664, AARCH64, ARMV7L, RISCV64, *y.Arch)
 	}
 
 	switch *y.VMType {
@@ -91,7 +91,7 @@ func Validate(y LimaYAML, warn bool) error {
 
 	for arch := range y.CPUType {
 		switch arch {
-		case AARCH64, X8664, RISCV64:
+		case AARCH64, X8664, ARMV7L, RISCV64:
 			// these are the only supported architectures
 		default:
 			return fmt.Errorf("field `cpuType` uses unsupported arch %q", arch)
@@ -159,6 +159,14 @@ func Validate(y LimaYAML, warn bool) error {
 		return fmt.Errorf("field `mountType` must be %q or %q or %q, got %q", REVSSHFS, NINEP, VIRTIOFS, *y.MountType)
 	}
 
+	if warn && runtime.GOOS != "linux" {
+		for i, mount := range y.Mounts {
+			if mount.Virtiofs.QueueSize != nil {
+				logrus.Warnf("field mounts[%d].virtiofs.queueSize is only supported on Linux", i)
+			}
+		}
+	}
+
 	// y.Firmware.LegacyBIOS is ignored for aarch64, but not a fatal error.
 
 	for i, p := range y.Provision {
@@ -442,6 +450,9 @@ func warnExperimental(y LimaYAML) {
 	if *y.MountType == NINEP {
 		logrus.Warn("`mountType: 9p` is experimental")
 	}
+	if *y.MountType == VIRTIOFS && runtime.GOOS == "linux" {
+		logrus.Warn("`mountType: virtiofs` on Linux is experimental")
+	}
 	if *y.VMType == VZ {
 		logrus.Warn("`vmType: vz` is experimental")
 	}
