@@ -2,6 +2,7 @@ package qemu
 
 import (
 	"bytes"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io/fs"
@@ -289,12 +290,16 @@ type features struct {
 	// e.g. "Supported machines are:\nakita...\n...virt-6.2...\n...virt-7.0...\n...\n"
 	// Not machine-readable, but checking strings.Contains() should be fine.
 	MachineHelp []byte
+	// CPUHelp is the output of `qemu-system-x86_64 -cpu help`
+	// e.g. "Available CPUs:\n...\nx86 base...\nx86 host...\n...\n"
+	// Not machine-readable, but checking strings.Contains() should be fine.
+	CPUHelp []byte
 
 	// VersionGEQ7 is true when the QEMU version seems v7.0.0 or later
 	VersionGEQ7 bool
 }
 
-func inspectFeatures(exe string) (*features, error) {
+func inspectFeatures(exe string, machine string) (*features, error) {
 	var (
 		f      features
 		stdout bytes.Buffer
@@ -337,6 +342,19 @@ func inspectFeatures(exe string) (*features, error) {
 	}
 	f.VersionGEQ7 = strings.Contains(string(f.MachineHelp), "-7.0")
 
+	// Avoid error: "No machine specified, and there is no default"
+	cmd = exec.Command(exe, "-cpu", "help", "-machine", machine)
+	cmd.Stdout = &stdout
+	cmd.Stderr = &stderr
+	if err := cmd.Run(); err != nil {
+		logrus.Warnf("failed to run %v: stdout=%q, stderr=%q", cmd.Args, stdout.String(), stderr.String())
+	} else {
+		f.CPUHelp = stdout.Bytes()
+		if len(f.CPUHelp) == 0 {
+			f.CPUHelp = stderr.Bytes()
+		}
+	}
+
 	return &f, nil
 }
 
@@ -441,6 +459,14 @@ func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string, features *featur
 	return memBytes
 }
 
+// qemuMachine returns string to use for -machine
+func qemuMachine(arch limayaml.Arch) string {
+	if arch == limayaml.X8664 {
+		return "q35"
+	}
+	return "virt"
+}
+
 func Cmdline(cfg Config) (string, []string, error) {
 	y := cfg.LimaYAML
 	exe, args, err := getExe(*y.Arch)
@@ -448,7 +474,7 @@ func Cmdline(cfg Config) (string, []string, error) {
 		return "", nil, err
 	}
 
-	features, err := inspectFeatures(exe)
+	features, err := inspectFeatures(exe, qemuMachine(*y.Arch))
 	if err != nil {
 		return "", nil, err
 	}
@@ -478,6 +504,12 @@ func Cmdline(cfg Config) (string, []string, error) {
 	memBytes = adjustMemBytesDarwinARM64HVF(memBytes, accel, features)
 	args = appendArgsIfNoConflict(args, "-m", strconv.Itoa(int(memBytes>>20)))
 
+	if *y.MountType == limayaml.VIRTIOFS {
+		args = appendArgsIfNoConflict(args, "-object",
+			fmt.Sprintf("memory-backend-file,id=virtiofs-shm,size=%s,mem-path=/dev/shm,share=on", strconv.Itoa(int(memBytes))))
+		args = appendArgsIfNoConflict(args, "-numa", "node,memdev=virtiofs-shm")
+	}
+
 	// CPU
 	cpu := y.CPUType[*y.Arch]
 	if runtime.GOOS == "darwin" && runtime.GOARCH == "amd64" {
@@ -488,8 +520,12 @@ func Cmdline(cfg Config) (string, []string, error) {
 			}
 		}
 	}
+	if !strings.Contains(string(features.CPUHelp), strings.Split(cpu, ",")[0]) {
+		return "", nil, fmt.Errorf("cpu %q is not supported by %s", cpu, exe)
+	}
 	args = appendArgsIfNoConflict(args, "-cpu", cpu)
 
+	// Machine
 	switch *y.Arch {
 	case limayaml.X8664:
 		if strings.HasPrefix(cpu, "qemu64") && runtime.GOOS != "windows" {
@@ -523,6 +559,9 @@ func Cmdline(cfg Config) (string, []string, error) {
 	case limayaml.RISCV64:
 		machine := "virt,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
+	case limayaml.ARMV7L:
+		machine := "virt,accel=" + accel
+		args = appendArgsIfNoConflict(args, "-machine", machine)
 	}
 
 	// SMP
@@ -531,7 +570,7 @@ func Cmdline(cfg Config) (string, []string, error) {
 
 	// Firmware
 	legacyBIOS := *y.Firmware.LegacyBIOS
-	if legacyBIOS && *y.Arch != limayaml.X8664 {
+	if legacyBIOS && *y.Arch != limayaml.X8664 && *y.Arch != limayaml.ARMV7L {
 		logrus.Warnf("field `firmware.legacyBIOS` is not supported for architecture %q, ignoring", *y.Arch)
 		legacyBIOS = false
 	}
@@ -548,24 +587,25 @@ func Cmdline(cfg Config) (string, []string, error) {
 	diffDisk := filepath.Join(cfg.InstanceDir, filenames.DiffDisk)
 	extraDisks := []string{}
 	if len(y.AdditionalDisks) > 0 {
-		for _, diskName := range y.AdditionalDisks {
-			d, err := store.InspectDisk(diskName)
+		for _, d := range y.AdditionalDisks {
+			diskName := d.Name
+			disk, err := store.InspectDisk(diskName)
 			if err != nil {
 				logrus.Errorf("could not load disk %q: %q", diskName, err)
 				return "", nil, err
 			}
 
-			if d.Instance != "" {
-				logrus.Errorf("could not attach disk %q, in use by instance %q", diskName, d.Instance)
+			if disk.Instance != "" {
+				logrus.Errorf("could not attach disk %q, in use by instance %q", diskName, disk.Instance)
 				return "", nil, err
 			}
-			logrus.Infof("Mounting disk %q on %q", diskName, d.MountPoint)
-			err = d.Lock(cfg.InstanceDir)
+			logrus.Infof("Mounting disk %q on %q", diskName, disk.MountPoint)
+			err = disk.Lock(cfg.InstanceDir)
 			if err != nil {
 				logrus.Errorf("could not lock disk %q: %q", diskName, err)
 				return "", nil, err
 			}
-			dataDisk := filepath.Join(d.Dir, filenames.DataDisk)
+			dataDisk := filepath.Join(disk.Dir, filenames.DataDisk)
 			extraDisks = append(extraDisks, dataDisk)
 		}
 	}
@@ -749,8 +789,8 @@ func Cmdline(cfg Config) (string, []string, error) {
 		args = append(args, "-device", "virtio-keyboard-pci")
 		args = append(args, "-device", "virtio-mouse-pci")
 		args = append(args, "-device", "qemu-xhci,id=usb-bus")
-	default:
-		// QEMU does not seem to support virtio-vga for aarch64
+	case limayaml.AARCH64, limayaml.ARMV7L:
+		// QEMU does not seem to support virtio-vga for aarch64 and arm
 		args = append(args, "-vga", "none", "-device", "ramfb")
 		args = append(args, "-device", "qemu-xhci,id=usb-bus")
 		args = append(args, "-device", "usb-kbd,bus=usb-bus.0")
@@ -760,7 +800,8 @@ func Cmdline(cfg Config) (string, []string, error) {
 	// Parallel
 	args = append(args, "-parallel", "none")
 
-	// Serial
+	// Serial (default)
+	// This is ttyS0 for Intel and RISC-V, ttyAMA0 for ARM.
 	serialSock := filepath.Join(cfg.InstanceDir, filenames.SerialSock)
 	if err := os.RemoveAll(serialSock); err != nil {
 		return "", nil, err
@@ -773,9 +814,42 @@ func Cmdline(cfg Config) (string, []string, error) {
 	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialChardev, serialSock, serialLog))
 	args = append(args, "-serial", "chardev:"+serialChardev)
 
+	// Serial (PCI, ARM only)
+	// On ARM, the default serial is ttyAMA0, this PCI serial is ttyS0.
+	// https://gitlab.com/qemu-project/qemu/-/issues/1801#note_1494720586
+	switch *y.Arch {
+	case limayaml.AARCH64, limayaml.ARMV7L:
+		serialpSock := filepath.Join(cfg.InstanceDir, filenames.SerialPCISock)
+		if err := os.RemoveAll(serialpSock); err != nil {
+			return "", nil, err
+		}
+		serialpLog := filepath.Join(cfg.InstanceDir, filenames.SerialPCILog)
+		if err := os.RemoveAll(serialpLog); err != nil {
+			return "", nil, err
+		}
+		const serialpChardev = "char-serial-pci"
+		args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialpChardev, serialpSock, serialpLog))
+		args = append(args, "-device", "pci-serial,chardev="+serialpChardev)
+	}
+
+	// Serial (virtio)
+	serialvSock := filepath.Join(cfg.InstanceDir, filenames.SerialVirtioSock)
+	if err := os.RemoveAll(serialvSock); err != nil {
+		return "", nil, err
+	}
+	serialvLog := filepath.Join(cfg.InstanceDir, filenames.SerialVirtioLog)
+	if err := os.RemoveAll(serialvLog); err != nil {
+		return "", nil, err
+	}
+	const serialvChardev = "char-serial-virtio"
+	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialvChardev, serialvSock, serialvLog))
+	// max_ports=1 is required for https://github.com/lima-vm/lima/issues/1689 https://github.com/lima-vm/lima/issues/1691
+	args = append(args, "-device", "virtio-serial-pci,id=virtio-serial0,max_ports=1")
+	args = append(args, "-device", fmt.Sprintf("virtconsole,chardev=%s,id=console0", serialvChardev))
+
 	// We also want to enable vsock here, but QEMU does not support vsock for macOS hosts
 
-	if *y.MountType == limayaml.NINEP {
+	if *y.MountType == limayaml.NINEP || *y.MountType == limayaml.VIRTIOFS {
 		for i, f := range y.Mounts {
 			tag := fmt.Sprintf("mount%d", i)
 			location, err := localpathutil.Expand(f.Location)
@@ -785,14 +859,30 @@ func Cmdline(cfg Config) (string, []string, error) {
 			if err := os.MkdirAll(location, 0755); err != nil {
 				return "", nil, err
 			}
-			options := "local"
-			options += fmt.Sprintf(",mount_tag=%s", tag)
-			options += fmt.Sprintf(",path=%s", location)
-			options += fmt.Sprintf(",security_model=%s", *f.NineP.SecurityModel)
-			if !*f.Writable {
-				options += ",readonly"
+
+			switch *y.MountType {
+			case limayaml.NINEP:
+				options := "local"
+				options += fmt.Sprintf(",mount_tag=%s", tag)
+				options += fmt.Sprintf(",path=%s", location)
+				options += fmt.Sprintf(",security_model=%s", *f.NineP.SecurityModel)
+				if !*f.Writable {
+					options += ",readonly"
+				}
+				args = append(args, "-virtfs", options)
+			case limayaml.VIRTIOFS:
+				// Note that read-only mode is not supported on the QEMU/virtiofsd side yet:
+				// https://gitlab.com/virtio-fs/virtiofsd/-/issues/97
+				chardev := fmt.Sprintf("char-virtiofs-%d", i)
+				vhostSock := filepath.Join(cfg.InstanceDir, fmt.Sprintf(filenames.VhostSock, i))
+				args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s", chardev, vhostSock))
+
+				options := "vhost-user-fs-pci"
+				options += fmt.Sprintf(",queue-size=%d", *f.Virtiofs.QueueSize)
+				options += fmt.Sprintf(",chardev=%s", chardev)
+				options += fmt.Sprintf(",tag=%s", tag)
+				args = append(args, "-device", options)
 			}
-			args = append(args, "-virtfs", options)
 		}
 	}
 
@@ -812,10 +902,111 @@ func Cmdline(cfg Config) (string, []string, error) {
 	return exe, args, nil
 }
 
+func FindVirtiofsd(qemuExe string) (string, error) {
+	type vhostUserBackend struct {
+		BackendType string `json:"type"`
+		Binary      string `json:"binary"`
+	}
+
+	currentUser, err := user.Current()
+	if err != nil {
+		return "", err
+	}
+
+	const relativePath = "share/qemu/vhost-user"
+
+	binDir := filepath.Dir(qemuExe)                              // "/usr/local/bin"
+	usrDir := filepath.Dir(binDir)                               // "/usr/local"
+	userLocalDir := filepath.Join(currentUser.HomeDir, ".local") // "$HOME/.local"
+
+	candidates := []string{
+		filepath.Join(userLocalDir, relativePath),
+		filepath.Join(usrDir, relativePath),
+	}
+
+	if usrDir != "/usr" {
+		candidates = append(candidates, filepath.Join("/usr", relativePath))
+	}
+
+	for _, vhostConfigsDir := range candidates {
+		logrus.Debugf("Checking vhost directory %s", vhostConfigsDir)
+
+		configEntries, err := os.ReadDir(vhostConfigsDir)
+		if err != nil {
+			logrus.Debugf("Failed to list vhost directory: %v", err)
+			continue
+		}
+
+		for _, configEntry := range configEntries {
+			logrus.Debugf("Checking vhost config %s", configEntry.Name())
+			if !strings.HasSuffix(configEntry.Name(), ".json") {
+				continue
+			}
+
+			var config vhostUserBackend
+			contents, err := os.ReadFile(filepath.Join(vhostConfigsDir, configEntry.Name()))
+			if err == nil {
+				err = json.Unmarshal(contents, &config)
+			}
+
+			if err != nil {
+				logrus.Warnf("Failed to load vhost-user config %s: %v", configEntry.Name(), err)
+				continue
+			}
+			logrus.Debugf("%v", config)
+
+			if config.BackendType != "fs" {
+				continue
+			}
+
+			// Only rust virtiofsd supports --version, so use that to make sure this isn't
+			// QEMU's virtiofsd, which requires running as root.
+			cmd := exec.Command(config.Binary, "--version")
+			output, err := cmd.CombinedOutput()
+			if err != nil {
+				logrus.Warnf("Failed to run %s --version (is this QEMU virtiofsd?): %s: %s",
+					config.Binary, err, output)
+				continue
+			}
+
+			return config.Binary, nil
+		}
+	}
+
+	return "", errors.New("Failed to locate virtiofsd")
+}
+
+func VirtiofsdCmdline(cfg Config, mountIndex int) ([]string, error) {
+	mount := cfg.LimaYAML.Mounts[mountIndex]
+	location, err := localpathutil.Expand(mount.Location)
+	if err != nil {
+		return nil, err
+	}
+
+	vhostSock := filepath.Join(cfg.InstanceDir, fmt.Sprintf(filenames.VhostSock, mountIndex))
+	// qemu_driver has to wait for the socket to appear, so make sure any old ones are removed here.
+	if err := os.Remove(vhostSock); err != nil && !errors.Is(err, fs.ErrNotExist) {
+		logrus.Warnf("Failed to remove old vhost socket: %v", err)
+	}
+
+	return []string{
+		"--socket-path", vhostSock,
+		"--shared-dir", location,
+	}, nil
+}
+
+// qemuArch returns the arch string used by qemu
+func qemuArch(arch limayaml.Arch) string {
+	if arch == limayaml.ARMV7L {
+		return "arm"
+	}
+	return arch
+}
+
 func getExe(arch limayaml.Arch) (string, []string, error) {
-	exeBase := "qemu-system-" + arch
+	exeBase := "qemu-system-" + qemuArch(arch)
 	var args []string
-	envK := "QEMU_SYSTEM_" + strings.ToUpper(arch)
+	envK := "QEMU_SYSTEM_" + strings.ToUpper(qemuArch(arch))
 	if envV := os.Getenv(envK); envV != "" {
 		ss, err := shellwords.Parse(envV)
 		if err != nil {
@@ -876,7 +1067,7 @@ func getQemuVersion(qemuExe string) (*semver.Version, error) {
 
 func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 	switch arch {
-	case limayaml.X8664, limayaml.AARCH64:
+	case limayaml.X8664, limayaml.AARCH64, limayaml.ARMV7L:
 	default:
 		return "", fmt.Errorf("unexpected architecture: %q", arch)
 	}
@@ -890,7 +1081,7 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 	localDir := filepath.Dir(binDir)                             // "/usr/local"
 	userLocalDir := filepath.Join(currentUser.HomeDir, ".local") // "$HOME/.local"
 
-	relativePath := fmt.Sprintf("share/qemu/edk2-%s-code.fd", arch)
+	relativePath := fmt.Sprintf("share/qemu/edk2-%s-code.fd", qemuArch(arch))
 	candidates := []string{
 		filepath.Join(userLocalDir, relativePath), // XDG-like
 		filepath.Join(localDir, relativePath),     // macOS (homebrew)
@@ -909,6 +1100,9 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		candidates = append(candidates, "/usr/share/AAVMF/AAVMF_CODE.fd")
 		// Debian package "qemu-efi-aarch64" (unpadded, backwards compatibility)
 		candidates = append(candidates, "/usr/share/qemu-efi-aarch64/QEMU_EFI.fd")
+	case limayaml.ARMV7L:
+		// Debian package "qemu-efi-arm"
+		candidates = append(candidates, "/usr/share/AAVMF/AAVMF32_CODE.fd")
 	}
 
 	logrus.Debugf("firmware candidates = %v", candidates)
