@@ -12,19 +12,20 @@ import (
 	"path/filepath"
 	"runtime"
 	"strconv"
-	"strings"
 	"sync"
 	"syscall"
 
 	"github.com/Code-Hex/vz/v3"
 	"github.com/docker/go-units"
+	"github.com/lima-vm/go-qcow2reader"
+	"github.com/lima-vm/go-qcow2reader/image/raw"
 	"github.com/lima-vm/lima/pkg/driver"
 	"github.com/lima-vm/lima/pkg/iso9660util"
 	"github.com/lima-vm/lima/pkg/limayaml"
 	"github.com/lima-vm/lima/pkg/localpathutil"
+	"github.com/lima-vm/lima/pkg/nativeimgutil"
 	"github.com/lima-vm/lima/pkg/networks"
 	"github.com/lima-vm/lima/pkg/networks/usernet"
-	"github.com/lima-vm/lima/pkg/qemu/imgutil"
 	"github.com/lima-vm/lima/pkg/store"
 	"github.com/lima-vm/lima/pkg/store/filenames"
 	"github.com/sirupsen/logrus"
@@ -58,6 +59,13 @@ func startVM(ctx context.Context, driver *driver.BaseDriver) (*virtualMachineWra
 	wrapper := &virtualMachineWrapper{VirtualMachine: machine, stopped: false}
 
 	errCh := make(chan error)
+
+	filesToRemove := make(map[string]struct{})
+	defer func() {
+		for f := range filesToRemove {
+			_ = os.RemoveAll(f)
+		}
+	}()
 	go func() {
 		//Handle errors via errCh and handle stop vm during context close
 		defer func() {
@@ -85,7 +93,7 @@ func startVM(ctx context.Context, driver *driver.BaseDriver) (*virtualMachineWra
 						logrus.Errorf("error writing to pid fil %q", pidFile)
 						errCh <- err
 					}
-					defer os.RemoveAll(pidFile)
+					filesToRemove[pidFile] = struct{}{}
 					logrus.Info("[VZ] - vm state change: running")
 
 					err := usernetClient.ResolveAndForwardSSH(limayaml.MACAddress(driver.Instance.Dir), driver.SSHLocalPort)
@@ -113,15 +121,24 @@ func startUsernet(ctx context.Context, driver *driver.BaseDriver) (*usernet.Clie
 	firstUsernetIndex := limayaml.FirstUsernetIndex(driver.Yaml)
 	if firstUsernetIndex == -1 {
 		//Start a in-process gvisor-tap-vsock
-		endpointSock := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.EndpointSock)
-		vzSock := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.FDSock)
+		endpointSock, err := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.EndpointSock)
+		if err != nil {
+			return nil, err
+		}
+		vzSock, err := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.FDSock)
+		if err != nil {
+			return nil, err
+		}
 		os.RemoveAll(endpointSock)
 		os.RemoveAll(vzSock)
-		err := usernet.StartGVisorNetstack(ctx, &usernet.GVisorNetstackOpts{
+		err = usernet.StartGVisorNetstack(ctx, &usernet.GVisorNetstackOpts{
 			MTU:      1500,
 			Endpoint: endpointSock,
 			FdSocket: vzSock,
 			Async:    true,
+			DefaultLeases: map[string]string{
+				networks.SlirpIPAddress: limayaml.MACAddress(driver.Instance.Dir),
+			},
 		})
 		if err != nil {
 			return nil, err
@@ -223,7 +240,7 @@ func attachPlatformConfig(driver *driver.BaseDriver, vmConfig *vz.VirtualMachine
 }
 
 func attachSerialPort(driver *driver.BaseDriver, config *vz.VirtualMachineConfiguration) error {
-	path := filepath.Join(driver.Instance.Dir, filenames.SerialLog)
+	path := filepath.Join(driver.Instance.Dir, filenames.SerialVirtioLog)
 	serialPortAttachment, err := vz.NewFileSerialPortAttachment(path, false)
 	if err != nil {
 		return err
@@ -267,7 +284,10 @@ func attachNetwork(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfigu
 	firstUsernetIndex := limayaml.FirstUsernetIndex(driver.Yaml)
 	if firstUsernetIndex == -1 {
 		//slirp network using gvisor netstack
-		vzSock := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.FDSock)
+		vzSock, err := usernet.SockWithDirectory(driver.Instance.Dir, "", usernet.FDSock)
+		if err != nil {
+			return err
+		}
 		networkConn, err := PassFDToUnix(vzSock)
 		if err != nil {
 			return err
@@ -373,12 +393,17 @@ func attachNetwork(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfigu
 }
 
 func validateDiskFormat(diskPath string) error {
-	info, err := imgutil.GetInfo(diskPath)
+	f, err := os.Open(diskPath)
+	if err != nil {
+		return err
+	}
+	defer f.Close()
+	img, err := qcow2reader.Open(f)
 	if err != nil {
 		return fmt.Errorf("failed to detect the format of %q: %w", diskPath, err)
 	}
-	if info.Format != "raw" {
-		return fmt.Errorf("expected the format of %q to be \"raw\", got %q", diskPath, info.Format)
+	if t := img.Type(); t != raw.Type {
+		return fmt.Errorf("expected the format of %q to be %q, got %q", diskPath, raw.Type, t)
 	}
 	// TODO: ensure that the disk is formatted with GPT or ISO9660
 	return nil
@@ -421,54 +446,26 @@ func attachDisks(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfigura
 	}
 	configurations = append(configurations, diffDisk)
 
-	for _, diskName := range driver.Yaml.AdditionalDisks {
-		d, err := store.InspectDisk(diskName)
+	for _, d := range driver.Yaml.AdditionalDisks {
+		diskName := d.Name
+		disk, err := store.InspectDisk(diskName)
 		if err != nil {
 			return fmt.Errorf("failed to run load disk %q: %q", diskName, err)
 		}
 
-		if d.Instance != "" {
-			return fmt.Errorf("failed to run attach disk %q, in use by instance %q", diskName, d.Instance)
+		if disk.Instance != "" {
+			return fmt.Errorf("failed to run attach disk %q, in use by instance %q", diskName, disk.Instance)
 		}
-		logrus.Infof("Mounting disk %q on %q", diskName, d.MountPoint)
-		err = d.Lock(driver.Instance.Dir)
+		logrus.Infof("Mounting disk %q on %q", diskName, disk.MountPoint)
+		err = disk.Lock(driver.Instance.Dir)
 		if err != nil {
 			return fmt.Errorf("failed to run lock disk %q: %q", diskName, err)
 		}
-		extraDiskPath := filepath.Join(d.Dir, filenames.DataDisk)
-
-		extraDiskInfo, err := imgutil.GetInfo(extraDiskPath)
-		if err != nil {
-			return fmt.Errorf("failed to run detect disk format %q: %q", diskName, err)
-		}
-		if extraDiskInfo.Format != "raw" {
-			if strings.Contains(extraDiskInfo.Format, string(os.PathSeparator)) {
-				return fmt.Errorf(
-					"failed to convert disk %q to raw for vz driver because extraDiskFormat %q contains a path separator %q",
-					diskName,
-					extraDiskInfo.Format,
-					os.PathSeparator,
-				)
-			}
-			rawPath := fmt.Sprintf("%s.raw", extraDiskPath)
-			oldFormatPath := fmt.Sprintf("%s.%s", extraDiskPath, extraDiskInfo.Format)
-			if err = imgutil.ConvertToRaw(extraDiskPath, rawPath); err != nil {
-				return fmt.Errorf("failed to convert %s disk %q to raw for vz driver: %w", extraDiskInfo.Format, diskName, err)
-			}
-			if err = os.Rename(extraDiskPath, oldFormatPath); err != nil {
-				return fmt.Errorf("failed to rename additional disk for vz driver: %w", err)
-			}
-			if err = os.Rename(rawPath, extraDiskPath); err != nil {
-				return fmt.Errorf("failed to rename additional disk for vz driver: %w", err)
-			}
-			if err = os.Remove(oldFormatPath); err != nil {
-				logrus.Errorf("Failed to delete unused qcow2 additional disk %q."+
-					"Disk is no longer needed by Lima and it can be removed manually.", oldFormatPath)
-			}
-		}
-
-		if err = validateDiskFormat(extraDiskPath); err != nil {
-			return fmt.Errorf("failed to validate extra disk %q: %w", extraDiskPath, err)
+		extraDiskPath := filepath.Join(disk.Dir, filenames.DataDisk)
+		// ConvertToRaw is a NOP if no conversion is needed
+		logrus.Debugf("Converting extra disk %q to a raw disk (if it is not a raw)", extraDiskPath)
+		if err = nativeimgutil.ConvertToRaw(extraDiskPath, extraDiskPath, nil, true); err != nil {
+			return fmt.Errorf("failed to convert extra disk %q to a raw disk: %w", extraDiskPath, err)
 		}
 		extraDiskPathAttachment, err := vz.NewDiskImageStorageDeviceAttachmentWithCacheAndSync(extraDiskPath, false, vz.DiskImageCachingModeAutomatic, vz.DiskImageSynchronizationModeFsync)
 		if err != nil {
@@ -499,7 +496,8 @@ func attachDisks(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfigura
 }
 
 func attachDisplay(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfiguration) error {
-	if driver.Yaml.Video.Display != nil {
+	switch *driver.Yaml.Video.Display {
+	case "vz", "default":
 		graphicsDeviceConfiguration, err := vz.NewVirtioGraphicsDeviceConfiguration()
 		if err != nil {
 			return err
@@ -513,8 +511,12 @@ func attachDisplay(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfigu
 		vmConfig.SetGraphicsDevicesVirtualMachineConfiguration([]vz.GraphicsDeviceConfiguration{
 			graphicsDeviceConfiguration,
 		})
+		return nil
+	case "none":
+		return nil
+	default:
+		return fmt.Errorf("unexpected video display %q", *driver.Yaml.Video.Display)
 	}
-	return nil
 }
 
 func attachFolderMounts(driver *driver.BaseDriver, vmConfig *vz.VirtualMachineConfiguration) error {
