@@ -14,7 +14,6 @@ import (
 
 	"github.com/Code-Hex/vz/v3"
 
-	"github.com/lima-vm/lima/pkg/store/filenames"
 	"github.com/sirupsen/logrus"
 
 	"github.com/lima-vm/lima/pkg/driver"
@@ -60,6 +59,7 @@ func (l *LimaVzDriver) Validate() error {
 		"Firmware",
 		"Provision",
 		"Containerd",
+		"GuestInstallPrefix",
 		"Probes",
 		"PortForwards",
 		"Message",
@@ -72,6 +72,7 @@ func (l *LimaVzDriver) Validate() error {
 		"Rosetta",
 		"AdditionalDisks",
 		"Audio",
+		"Video",
 	); len(unknown) > 0 {
 		logrus.Warnf("Ignoring: vmType %s: %+v", *l.Yaml.VMType, unknown)
 	}
@@ -118,19 +119,21 @@ func (l *LimaVzDriver) Validate() error {
 	if audioDevice != "" && audioDevice != "vz" {
 		logrus.Warnf("field `audio.device` must be %q for VZ driver , got %q", "vz", audioDevice)
 	}
+
+	switch videoDisplay := *l.Yaml.Video.Display; videoDisplay {
+	case "vz", "default", "none":
+	default:
+		logrus.Warnf("field `video.display` must be \"vz\", \"default\", or \"none\" for VZ driver , got %q", videoDisplay)
+	}
 	return nil
 }
 
 func (l *LimaVzDriver) CreateDisk() error {
-	if err := EnsureDisk(l.BaseDriver); err != nil {
-		return err
-	}
-
-	return nil
+	return EnsureDisk(l.BaseDriver)
 }
 
 func (l *LimaVzDriver) Start(ctx context.Context) (chan error, error) {
-	logrus.Infof("Starting VZ (hint: to watch the boot progress, see %q)", filepath.Join(l.Instance.Dir, filenames.SerialLog))
+	logrus.Infof("Starting VZ (hint: to watch the boot progress, see %q)", filepath.Join(l.Instance.Dir, "serial*.log"))
 	vm, errCh, err := startVM(ctx, l.BaseDriver)
 	if err != nil {
 		if errors.Is(err, vz.ErrUnsupportedOSVersion) {
@@ -143,6 +146,22 @@ func (l *LimaVzDriver) Start(ctx context.Context) (chan error, error) {
 	return errCh, nil
 }
 
+func (l *LimaVzDriver) CanRunGUI() bool {
+	switch *l.Yaml.Video.Display {
+	case "vz", "default":
+		return true
+	default:
+		return false
+	}
+}
+
+func (l *LimaVzDriver) RunGUI() error {
+	if l.CanRunGUI() {
+		return l.machine.StartGraphicApplication(1920, 1200)
+	}
+	return fmt.Errorf("RunGUI is not support for the given driver '%s' and diplay '%s'", "vz", *l.Yaml.Video.Display)
+}
+
 func (l *LimaVzDriver) Stop(_ context.Context) error {
 	logrus.Info("Shutting down VZ")
 	canStop := l.machine.CanRequestStop()
@@ -154,12 +173,12 @@ func (l *LimaVzDriver) Stop(_ context.Context) error {
 		}
 
 		timeout := time.After(5 * time.Second)
-		tick := time.Tick(500 * time.Millisecond)
+		ticker := time.NewTicker(500 * time.Millisecond)
 		for {
 			select {
 			case <-timeout:
 				return errors.New("vz timeout while waiting for stop status")
-			case <-tick:
+			case <-ticker.C:
 				l.machine.mu.Lock()
 				stopped := l.machine.stopped
 				l.machine.mu.Unlock()
